/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/liberty-rtk-addon/c++/org/src/main_protocoltest.cc,v 1.2 2003/07/22 21:06:32 tubadanm Exp $ */

#include <string>
#include <iostream>

#include "epp-rtk-cpp/data/epp_eppdata.h"
#include "epp-rtk-cpp/data/epp_DomainInfoReq.h"
#include "epp-rtk-cpp/data/epp_DomainInfoRsp.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "OxrsProtocol.h"

using namespace std;
using namespace eppobject::domain;


void printRsp(const epp_DomainInfoRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart1(req->m_rsp);
      printResDataHeaders("domain", "infData");
      
      if(req->m_name != NULL)
        cerr << "<domain:name>" << *req-> m_name << "</domain:name>" << endl;
      
      if(req->m_roid != NULL)
	cerr << "<domain:roid>" << *req->m_roid << "</domain:roid>" << endl;
            
      printResDataFooter("domain", "infData");
      printResponsePart2(req->m_rsp);
   } // if(req != NULL)
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   eppobject::epp::epp_Session test;
   
   string username, password;
   
   string extra_usage = "[-extradata=trid=xxx:name=abc.tld";
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;

   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
            cout << "Found extra data param..." << endl;
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      
      epp_DomainInfoReq_ref l_req(new epp_DomainInfoReq());

      string trID;
      if(extraDataMap.empty()) trID = "ABC-12345";
      else trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
						       NULL,
						       eppobject::epp::epp_trid(trID)));
      
      if(extraDataMap.find("name") == extraDataMap.end()) {
	 l_req->m_name.ref(new eppobject::epp::epp_string("afilias.org"));
      }
      else {
	 l_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["name"]));
      }
      
      l_req->m_hosts_type.ref(new epp_DomainHostsType(ALL));
      
      epp_DomainInfo_ref l_obj(new epp_DomainInfo());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainInfoRsp_ref l_rsp = l_obj->getResponseData();
      
      eppobject::epp::epp_xml_string_seq_ref extension_strings;
      extension_strings = l_rsp->m_rsp->m_ext_strings;
      eppobject::epp::epp_xml_string first_extension = extension_strings->front();
      OxrsProtocol oxrsProtocol;
      oxrsProtocol.fromXML(first_extension);
      
      cout << "The domain's protocol is: "<<*oxrsProtocol.m_protocol<<endl;
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;
      
   }
   catch(const eppobject::epp::epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const eppobject::epp::epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
