/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry 
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainCheck.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"

#include "TestNumber.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCheckRsp_ref req)
{
    cout << "Printing Response:\n_______________________________\n";

    if(req != NULL) {
        printResponsePart(req->m_rsp);

        if(req->m_results != NULL) {
            for(epp_check_result_seq::iterator i =req->m_results->begin(); i != req->m_results->end(); i++)
            {
                string exists = *i->m_avail ? "Y" : "N";
                cout << "m_exists: " << exists << endl;
                cout << "m_value: " << *i->m_value << endl;
            }
        }
    }

    cout << "_______________________________\n";
} // printRsp

int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   if(!handleArgs(test, argc, argv, username, password))
     return 1;
   
   testLogIn(test, username, password);
   
   try {
       
      epp_DomainCheckReq_ref l_req(new epp_DomainCheckReq());
      
      TestNumber_ref testNumber(new TestNumber());
      testNumber->m_test_num.ref(new epp_string("2.6.11"));
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       testNumber,
				       epp_trid("ABC-12345")));
      
      epp_string_seq_ref names(new epp_string_seq());
      names->push_back("example1.info");
      names->push_back("example1.info");
      l_req->m_names = names;


      epp_DomainCheck_ref l_obj(new epp_DomainCheck());
      
      l_obj->setRequestData(*l_req);
      

      test.processAction(l_obj);
      
      epp_DomainCheckRsp_ref l_rsp = l_obj->getResponseData();
      
      printRsp(l_rsp);
      
      return 0;
      
    }
    catch(epp_XMLException ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) {
        cout << "Something else went wrong...\n";
    }
}
